
/* bookmarks.h */

#ifndef BOOKMARKS_H
#define BOOKMARKS_H

#include <interfaces/iplugin.h>
#include <interfaces/iuicontroller.h>

#include <QPushButton>
#include <QTreeWidget>

/* ----------------------------------------------------------------------- */

namespace BookmarkModule {

/* ----------------------------------------------------------------------- */

class BookmarkPlugin : public KDevelop::IPlugin
{
   Q_OBJECT

   public:
       BookmarkPlugin (QObject *parent, const QVariantList & args = QVariantList ());
       virtual ~ BookmarkPlugin ();

   public:
       virtual void unload ();

   private Q_SLOTS:
       void run ();

   private:
      KDevelop::IToolViewFactory * m_view_factory;
      KDevelop::IToolViewFactory * m_trace_factory;
};

/* ----------------------------------------------------------------------- */

class BookmarkViewFactory : public KDevelop::IToolViewFactory
{
   public:
       virtual QWidget * create (QWidget * parent = 0);
       virtual Qt::DockWidgetArea defaultPosition();
       virtual QString id() const;
   public:
       BookmarkViewFactory (BookmarkPlugin * plugin);
   private:
       BookmarkPlugin * m_plugin;
};

/* ----------------------------------------------------------------------- */

class BookmarkView : public QWidget
{
   Q_OBJECT

   public:
       explicit BookmarkView (BookmarkPlugin* plugin, QWidget *parent = 0 );
       // virtual ~BookmarkView ();

   private Q_SLOTS:
       void click ();
       void doubleClick (QTreeWidgetItem * item, int column);

   private:
       BookmarkPlugin * m_plugin;

       QPushButton * m_button;
       QTreeWidget * m_tree;
};

/* ----------------------------------------------------------------------- */

} // end of namespace

#endif // BOOKMARKS_H
