
/* trace.cc */

#include "traceview.h"
#include "trace.h"

#include <klocale.h>
#include <kiconloader.h>

#include <interfaces/iuicontroller.h>

namespace BookmarkModule {

/* ----------------------------------------------------------------------- */

class TraceFactory: public KDevelop::IToolViewFactory
{
   public:
      virtual QWidget* create (QWidget *parent = 0)
      {
          m_widget = new TraceWidget (parent);
          return m_widget;
      }

      virtual Qt::DockWidgetArea defaultPosition ()
      {
          return Qt::BottomDockWidgetArea;
      }

      virtual QString id () const
      {
          return "org.kdevelop.Bookmarks.Trace";
      }

   private:
      TraceWidget * m_widget;

   public:
      TraceFactory () : m_widget (NULL) { }
};

/* ----------------------------------------------------------------------- */

#if 0
class TraceFactory: public KDevelop::IToolViewFactory
{
   public:
      virtual QWidget* create (QWidget *parent = 0);
      virtual Qt::DockWidgetArea defaultPosition ();
      virtual QString id () const;
   private:
      TraceWidget * m_widget;
   public:
      TraceFactory ();
};

QWidget* TraceFactory::create (QWidget *parent)
{
   m_widget = new TraceWidget (parent);
   return m_widget;
}

Qt::DockWidgetArea TraceFactory::defaultPosition ()
{
   return Qt::BottomDockWidgetArea;
}

QString TraceFactory::id () const
{
   return "org.kdevelop.BookmarkList.Trace";
}

TraceFactory::TraceFactory () :
   m_widget (NULL)
{ }
#endif

/* ----------------------------------------------------------------------- */

static TraceWidget * g_trace  = NULL;
static QTreeWidgetItem * g_current = NULL;

/* ----------------------------------------------------------------------- */

TraceWidget::TraceWidget(QWidget* parent) :
   QWidget (parent),
   m_button1 (NULL),
   m_button2 (NULL),
   m_tree (NULL)
{
   setObjectName("Trace Tree");
   setWindowTitle(i18n("Trace"));
   setWindowIcon(SmallIcon("code-class"));

   m_tree = new QTreeWidget (this);
   m_tree->header()->setResizeMode(0, QHeaderView::ResizeToContents);
   // m_tree->header()->setStretchLastSection(false);

   connect (m_tree,
            SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)),
            this,
            SLOT(treeDoubleClick(QTreeWidgetItem*,int)));

   m_button1 = new QPushButton ("Button 1", this);
   connect (m_button1, SIGNAL (clicked ()), this, SLOT (firstButtonClick ()));

   m_button2 = new QPushButton ("Button 2", this);
   connect (m_button2, SIGNAL (clicked ()), this, SLOT (secondButtonClick ()));

   QVBoxLayout* vbox = new QVBoxLayout(this);
   vbox->setMargin(0);

   QHBoxLayout* hbox = new QHBoxLayout(this);
   vbox->addLayout (hbox);
   hbox->addWidget (m_button1);
   hbox->addWidget (m_button2);

   vbox->addWidget(m_tree);
   setLayout( vbox );

   setWhatsThis( i18n( "Trace" ) );

   m_action = new KAction (this);
   m_action->setText (i18n ("Action Click"));
   connect (m_action, SIGNAL (triggered (bool)), this, SLOT (actionClick ()));

   g_trace = this;
   g_current = NULL;

   #if 0
   trace_open ("branch");
   trace ("sub item");
   trace_close ();
   trace ("next item");
   trace_alert ("alert");
   #endif
}

TraceWidget::~TraceWidget()
{
   g_trace = NULL;
   g_current = NULL;
}

/* ----------------------------------------------------------------------- */

void TraceWidget::contextMenuEvent (QContextMenuEvent* e)
{
   QMenu * menu = new QMenu (this);

   // Context* c;
   // QList<ContextMenuExtension> extensions = ICore::self()->pluginController()->queryPluginsForContextMenuExtensions( c );
   // ContextMenuExtension::populateMenu(menu, extensions);

   menu->addAction (m_action);

   if (!menu->actions().isEmpty())
      menu->exec(QCursor::pos());
}

void TraceWidget::actionClick ()
{
   trace ("action click");
}

void TraceWidget::treeDoubleClick (QTreeWidgetItem* item, int column)
{
   trace ("tree double click");
}

void TraceWidget::firstButtonClick ()
{
   m_button1->setText ("Click 1");
   trace ("first buton click");
}

void TraceWidget::secondButtonClick ()
{
   m_button2->setText ("Click 2");
   trace ("second button click");
}

/* ----------------------------------------------------------------------- */

QTreeWidgetItem * TraceWidget::show (QTreeWidgetItem * branch, QString text)
{
   QTreeWidgetItem * item;
   if (branch == NULL)
   {
      item = new QTreeWidgetItem ();
      m_tree->addTopLevelItem (item);
   }
   else
   {
      item = new QTreeWidgetItem (branch);
   }

   item->setText (0, text);

   return item;
}

/* ----------------------------------------------------------------------- */

void trace_start (QString text)
{
   g_current = NULL;
   trace_open (text);
}

void trace_open (QString text)
{
   if (g_trace != NULL)
      g_current = g_trace->show (g_current, text);
}

void trace_close ()
{
   if (g_current != NULL)
      g_current = g_current->parent ();
}

void trace (QString text)
{
   if (g_trace != NULL)
      g_trace->show (g_current, text);
}

void trace_alert (QString text)
{
   if (g_trace != NULL)
      g_trace->show (NULL, text);
}

/* ----------------------------------------------------------------------- */

KDevelop::IToolViewFactory * new_trace_factory ()
{
   return new TraceFactory ();
}

/* ----------------------------------------------------------------------- */

} // end of namesopace

#include "traceview.moc"
